@@ -84,6 +84,9 @@ void charmap_magic(ASS_Library *library, FT_Face face)
 
 uint32_t ass_font_index_magic(FT_Face face, uint32_t symbol)
 {
+    if (!face->charmap)
+        return symbol;
+
     switch(face->charmap->encoding){
     case FT_ENCODING_MS_SYMBOL:
         return 0xF000 | symbol;
@@ -225,33 +228,39 @@ ASS_Font *ass_font_new(Cache *font_cache, ASS_Library *library,
                        FT_Library ftlibrary, ASS_FontSelector *fontsel,
                        ASS_FontDesc *desc)
 {
-    int error;
-    ASS_Font *fontp;
-    ASS_Font font;
+    ASS_Font *font;
+    if (ass_cache_get(font_cache, desc, &font)) {
+        if (font->desc.family)
+            return font;
+        ass_cache_dec_ref(font);
+        return NULL;
+    }
+    if (!font)
+        return NULL;
 
-    fontp = ass_cache_get(font_cache, desc);
-    if (fontp)
-        return fontp;
+    font->library = library;
+    font->ftlibrary = ftlibrary;
+    font->shaper_priv = NULL;
+    font->n_faces = 0;
+    ASS_FontDesc *new_desc = ass_cache_key(font);
+    font->desc.family = new_desc->family;
+    font->desc.bold = desc->bold;
+    font->desc.italic = desc->italic;
+    font->desc.vertical = desc->vertical;
 
-    font.library = library;
-    font.ftlibrary = ftlibrary;
-    font.shaper_priv = NULL;
-    font.n_faces = 0;
-    font.desc.family = strdup(desc->family);
-    font.desc.bold = desc->bold;
-    font.desc.italic = desc->italic;
-    font.desc.vertical = desc->vertical;
+    font->scale_x = font->scale_y = 1.;
+    font->v.x = font->v.y = 0;
+    font->size = 0.;
 
-    font.scale_x = font.scale_y = 1.;
-    font.v.x = font.v.y = 0;
-    font.size = 0.;
-
-    error = add_face(fontsel, &font, 0);
+    int error = add_face(fontsel, font, 0);
     if (error == -1) {
-        free(font.desc.family);
-        return 0;
-    } else
-        return ass_cache_put(font_cache, &font.desc, &font);
+        font->desc.family = NULL;
+        ass_cache_commit(font, 1);
+        ass_cache_dec_ref(font);
+        return NULL;
+    }
+    ass_cache_commit(font, 1);
+    return font;
 }
 
 /**
@@ -671,9 +680,9 @@ FT_Glyph ass_font_get_glyph(ASS_Font *font, uint32_t ch, int face_index,
 }
 
 /**
- * \brief Deallocate ASS_Font
+ * \brief Deallocate ASS_Font internals
  **/
-void ass_font_free(ASS_Font *font)
+void ass_font_clear(ASS_Font *font)
 {
     int i;
     if (font->shaper_priv)
@@ -683,7 +692,6 @@ void ass_font_free(ASS_Font *font)
             FT_Done_Face(font->faces[i]);
     }
     free(font->desc.family);
-    free(font);
 }
 
 /**
